# IMPORT PACKAGES
import numpy as np
import matplotlib.pyplot as plt
import math
import random
import scipy.io

## FOR USERS: INPUT THE PARAMETERS BELOW
print("\nWelcome to HKU iGEM team (2020) cre-loxp circuit database!")
print("\nOur system allows a population of cells to express two phenotypes (F cells, R cells) at a stable ratio over time, and we can control the ratio by changing the probability of phenotype switching among the two phenotypes.")
print("\nYou may input the F:R cell ratio you wish, along with F cell and R cell division time and we will return the probabilities of phenotype switching for F cells (pF) and R cells (pR).")
FRratio = float(input("\nPlease enter the F:R cell ratio you wish to obtain:\t"))
tf = int(input("Please enter the division time of F cells (min) [integers between 10-20]:\t")) # F cells division time, range=[10,20]
tr = int(input("Please enter the division time of R cells (min) [integers between 10-20]:\t")) # R cells division time, range=[10,20]

## DEFINE DATABASE PARAMETERS
tini = 10
tfin = 20
pini = 0.01
pfin = 0.10

st = tfin - tini + 1 # Number of time states simulated
sp = int(round((pfin - pini)/0.01+1))

# IMPORT DATABASE
matdata = scipy.io.loadmat('database.mat')
database = matdata['out'] # Dimensions: database[tf,tr,pf,pr][0][8 outputs]

# RUN TRACING ALGORITHM
diff = np.zeros((sp,sp))
for i in range(sp):
	for j in range(sp):
		diff[i,j] = abs(database[tf-tini,tr-tini,i,j][0][4] - FRratio)
location = np.where(diff == np.amin(diff))
pfIndex = location[0][0]
prIndex = location[1][0]
pfFin = pini + 0.01 * pfIndex
prFin = pini + 0.01 * prIndex
result = database[tf-tini,tr-tini,pfIndex,prIndex][0]

print(f"\nTo achieve an F:R ratio of {FRratio}:1 with tf = {tf} and tr = {tr}:")
print(f"You should use pF = {pfFin} and pR = {prFin}")
print(f"This would yield an F:R ratio of {result[4]} ± {result[5]}")
print(f"The percentage of F cells = {result[0]*100}% ± {result[1]*100}%")
print(f"The percentage of R cells = {result[2]*100}% ± {result[3]*100}%")

