# IMPORT PACKAGES
import numpy as np
import matplotlib.pyplot as plt
import math
import random

# INPUT THE VARIABLES BELOW. INPUT INTEGERS. (td > 1 for now please.)
nf0 = 1000 # Number of F cells at t=0
nr0 = 1000 # Number of R cells at t=0
tf = 18 # F cells division time
tr = 20 # R cells division time
tm = 15 # Mutant (M) cells division time     ### SEARCH "NON-MUTANT ADJUSTED"
#ps = 0.05 # Probability that a cell will switch phenotype every minute, must be < 1
pF = 0.05 # Probability that F cells will switch phenotype every minute
pR = 0.05 # Probability that R cells will switch phenotype every minute
pm = 0.01 # Probability that mutant cells will form upon phenotype switching
pk = 0.99 # Probability that kill switch works
tTotal = 500 # Total modelling duration
trials = 1000 # Number of trials

# DEFINE THE FUNCTION
def division(nf0,nr0,tf,tr,tm,pF,pR,pm,pk,tTotal):
		
	# DEFINE THE MATRIXES REPRESENTING THE STATES
	f = np.zeros((tf,tTotal+1),dtype=int) # Substates of F cells (column) over time (row)
	r = np.zeros((tr,tTotal+1),dtype=int) # Substates of R cells (column) over time (row)
	ft = np.zeros((tr,tTotal+1),dtype=int) # Substates of F-transition (Ft) cells over time 
	rt = np.zeros((tf,tTotal+1),dtype=int) # Substates of R-transition (Ft) cells over time
	m = np.zeros((tm,tTotal+1),dtype=int) # Substates of mutant cells over time
	### NOTE THAT f and r has tTotal+1 columns which col(0) = t(0), col(tTotal) = t(tTotal)
	
	#Set initial number of F cells and R cells
	randf = []
	randr = []
	for i in range(nf0):
		randf.append(random.randint(0, tf-1))
	for i in range(tf):
		f[i,0] = randf.count(i)
	for i in range(nr0):
		randr.append(random.randint(0, tr-1))
	for i in range(tr):
		r[i,0] = randr.count(i)
	
	# CALCULATE THE NUMBER OF F AND R CELLS OVER TIME AT DIFFERENT STATES
	for i in range(tTotal):
		
		# FIRST: COMPUTE THE NUMBER OF CELLS SWITCHING PHENOTYPE @ DIFFERENT STATES
		Bf = np.zeros(tf, dtype=int) # Define no. of F cells which will switch phenotype
		Bft = np.zeros(tr, dtype=int) # Define no. of Ft cells which will switch phenotype
		Br = np.zeros(tr, dtype=int) # Define no. of R cells which will switch phenotype
		Brt = np.zeros(tf, dtype=int) # Define no. of Rt cells which will switch phenotype
		
		for j in range(tf-1): # Calculate Bf
			Bf[j] = sum(np.random.binomial(f[j,i],pF,size=1))
		Bf[tf-1] = sum(np.random.binomial(2*f[tf-1,i],pF,size=1))
		for j in range(tr-1): # Calculate Bft
			Bft[j] = sum(np.random.binomial(ft[j,i],pR,size=1))
		Bft[tr-1] = sum(np.random.binomial(2*ft[tr-1,i],pR,size=1))
		for j in range(tr-1): # Calculate Br
			Br[j] = sum(np.random.binomial(r[j,i],pR,size=1))
		Br[tr-1] = sum(np.random.binomial(2*r[tr-1,i],pR,size=1))
		for j in range(tf-1): # Calculate Brt
			Brt[j] = sum(np.random.binomial(rt[j,i],pF,size=1))
		Brt[tf-1] = sum(np.random.binomial(2*rt[tf-1,i],pF,size=1))
		
		# SECOND: CALCULATE NUMBER OF MUTANT CELLS
		Mf = np.zeros(tf, dtype=int) # Define no. of mutant cells from F cells  
		Mft = np.zeros(tr, dtype=int) # Define no. of mutant cells from Ft cells
		Mr = np.zeros(tr, dtype=int) # Define no. of mutant cells from R cells
		Mrt = np.zeros(tf, dtype=int) # Define no. of mutant cells from Rt cells
		
		for j in range(tf-1): # Calculate Mf
			Mf[j] = sum(np.random.binomial(Bf[j],pm,size=1))
		Mf[tf-1] = sum(np.random.binomial(Bf[tf-1],pm,size=1))
		for j in range(tr-1): # Calculate Mft
			Mft[j] = sum(np.random.binomial(Bft[j],pm,size=1))
		Mft[tr-1] = sum(np.random.binomial(Bft[tr-1],pm,size=1))
		for j in range(tr-1): # Calculate Mr
			Mr[j] = sum(np.random.binomial(Br[j],pm,size=1))
		Mr[tr-1] = sum(np.random.binomial(Br[tr-1],pm,size=1))
		for j in range(tf-1): # Calculate Mrt
			Mrt[j] = sum(np.random.binomial(Brt[j],pm,size=1))
		Mrt[tf-1] = sum(np.random.binomial(Brt[tf-1],pm,size=1))
		
		Bf = Bf - Mf # Calculate new Bf
		Bft = Bft - Mft # Calculate new Bft
		Br = Br - Mr  # Calculate new Br
		Brt = Brt - Mrt # Calculate new Bft
		mtot = sum(Mf) + sum(Mft) + sum(Mr) + sum(Mrt) # Calculate sum of new mutant cells
		
		# Kill switch
		kill = sum(np.random.binomial(mtot,pk,size=1))
		mtot = mtot - kill
		
		# Calculate mutant cells over time
		for j in range(tm-1):
			m[j+1,i+1] = m[j,i]
		m[0,i+1] = 2*m[tm-1,i] + mtot
		
		# THIRD: COMPUTE THE NUMBER OF CELLS IN SUBSEQUENT TIMEFRAMES
		
		# NON-DIVIDING CELLS:
		for j in range(0,tf-1):
			f[j+1,i+1] = f[j,i] - Bf[j] + Brt[j]
			rt[j+1,i+1] = rt[j,i] - Brt[j] + Bf[j]
			#print(f[j+1,i+1],rt[j+1,i+1])
		for j in range(0,tr-1):
			r[j+1,i+1] = r[j,i] - Br[j] + Bft[j]
			ft[j+1,i+1] = ft[j,i] - Bft[j] + Br[j]
		# DIVIDING CELLS:
		f[0,i+1] = 2*f[tf-1,i] - Bf[tf-1] + Br[tr-1] + 2*ft[tr-1,i] - Bft[tr-1] + Brt[tf-1]
		r[0,i+1] = 2*r[tr-1,i] - Br[tr-1] + Bf[tf-1] + 2*rt[tf-1,i] - Brt[tf-1] + Bft[tr-1]
	# THIRD: CALCULATE THE TOTAL NUMBER OF F/R CELLS REGARDLESS OF THEIR STATES
	fTotal = np.sum(f,axis=0) + np.sum(ft,axis=0) # Total number of F cells over time
	rTotal = np.sum(r,axis=0) + np.sum(rt,axis=0) # Total number of R cells over time
	mTotal = np.sum(m,axis=0) # Total number of mutant cells over time
	frTotal = fTotal + rTotal # Total number of non-mutant cells over time
	
	return fTotal, rTotal, mTotal, frTotal # The function should return these 3 values

### RUNNING THE SIMULATION

# RUN THE SIMULATION
fTotalMean = np.zeros((trials,tTotal+1))
rTotalMean = np.zeros((trials,tTotal+1))
mTotalMean = np.zeros((trials,tTotal+1))
frTotalMean = np.zeros((trials,tTotal+1))
print("\nSIMULATION RUNNING IN PROGRESS:\n")
for i in range(trials):
	output = division(nf0,nr0,tf,tr,tm,pF,pR,pm,pk,tTotal)
	fTotalMean[i,:] = output[0]
	rTotalMean[i,:] = output[1]
	mTotalMean[i,:] = output[2]
	frTotalMean[i,:] = output[3]
	print(f"LOADING: {i+1}/{trials} trials completed.")

# COMPUTE THE FINAL VALUES OF F CELLS, R CELLS, AND TOTAL NO. OF CELLS
fFinal = np.zeros((4,tTotal+1)) 
fFinal[0,:] = np.mean(fTotalMean,axis=0) # Row 1: Mean
fFinal[1,:] = np.std(fTotalMean,axis=0) # Row 2: SD
fFinal[2,:] = fFinal[0,:] - np.percentile(fTotalMean, 2.5, axis=0) # Row 3: 2.5% percentile -> Lower error bar
fFinal[3,:] = np.percentile(fTotalMean, 97.5,axis=0) - fFinal[0,:] # Row 4: 97.5% percentile -> Upper error bar
rFinal = np.zeros((4,tTotal+1)) 
rFinal[0,:] = np.mean(rTotalMean,axis=0)
rFinal[1,:] = np.std(rTotalMean,axis=0)
rFinal[2,:] = rFinal[0,:] - np.percentile(rTotalMean,2.5, axis=0)
rFinal[3,:] = np.percentile(rTotalMean, 97.5, axis=0) - rFinal[0,:]
mFinal = np.zeros((4,tTotal+1)) 
mFinal[0,:] = np.mean(mTotalMean,axis=0)
mFinal[1,:] = np.std(mTotalMean,axis=0)
mFinal[2,:] = mFinal[0,:] - np.percentile(mTotalMean,2.5, axis=0)
mFinal[3,:] = np.percentile(mTotalMean, 97.5, axis=0) - mFinal[0,:]
frFinal = np.zeros((4,tTotal+1)) 
frFinal[0,:] = np.mean(frTotalMean,axis=0)
frFinal[1,:] = np.std(frTotalMean,axis=0)
frFinal[2,:] = frFinal[0,:] - np.percentile(frTotalMean, 2.5, axis=0)
frFinal[3,:] = np.percentile(frTotalMean, 97.5, axis=0) - frFinal[0,:]
totFinal = np.zeros((4,tTotal+1)) 
totFinal[0,:] = mFinal[0,:] + frFinal[0,:]
totFinal[1,:] = np.std(mTotalMean + frTotalMean, axis=0)
totFinal[2,:] = totFinal[0,:] - np.percentile(mTotalMean + frTotalMean, 2.5, axis=0)
totFinal[3,:] = np.percentile(mTotalMean + frTotalMean, 97.5, axis=0) - totFinal[0,:]

# FINDING THE CV OVER TIME
fCV = fFinal[1,:]/fFinal[0,:]
rCV = rFinal[1,:]/rFinal[0,:]
mCV = mFinal[1,:]/mFinal[0,:]
frCV = frFinal[1,:]/frFinal[0,:]
totCV = totFinal[1,:]/totFinal[0,:]

# FIND THE RATIO OF F CELLS : R CELLS OVER TIME
ratio = np.zeros((2,tTotal+1)) 
ratio[0,:] = fFinal[0,:]/rFinal[0,:]
ratio[1,:] = (fCV + rCV)*ratio[0,:] # Find the error using simple relative error method
fRatio = np.zeros((2,tTotal+1)) 
fRatio[0,:] = fFinal[0,:]/frFinal[0,:]
fRatio[1,:] = (fCV + frCV)*fRatio[0,:]
rRatio = np.zeros((2,tTotal+1)) 
rRatio[0,:] = rFinal[0,:]/frFinal[0,:]
rRatio[1,:] = (rCV + frCV)*rRatio[0,:]
mRatio = np.zeros((2,tTotal+1)) # Find the ratio of mutant cells : overall number of cells
mRatio[0,:] = mFinal[0,:]/totFinal[0,:]
mRatio[1,:] = (mCV + totCV)*mRatio[0,:]

### CURVE FITTING

# EXPONENTIAL FITTING OF MEAN VALUES OVER TIME
timeAxis = list(range(0,tTotal+1))
# F cells fitting
lnF = np.log(fFinal[0,:])
kF,cF = np.polyfit(timeAxis, lnF, 1)
aF = math.exp(cF)
fitF = np.zeros(tTotal+1)
for i in range(tTotal+1):
	fitF[i] = aF * math.exp(kF * i)
corrF = np.corrcoef(fFinal[0,:], fitF)
rsqF = corrF[0,1]**2
# R cells fitting
lnR = np.log(rFinal[0,:])
kR,cR = np.polyfit(timeAxis, lnR, 1)
aR = math.exp(cR)
fitR = np.zeros(tTotal+1)
for i in range(tTotal+1):
	fitR[i] = aR * math.exp(kR * i)
corrR = np.corrcoef(rFinal[0,:], fitR)
rsqR = corrR[0,1]**2
# Mutant cells fitting
lnM = np.log(mFinal[0,:])
kM,cM = np.polyfit(timeAxis, lnM, 1)
aM = math.exp(cM)
fitM = np.zeros(tTotal+1)
for i in range(tTotal+1):
	fitM[i] = aM * math.exp(kM * i)
corrM = np.corrcoef(mFinal[0,:], fitM)
rsqM = corrM[0,1]**2
# Total live cells (FR) fitting
lnFR = np.log(frFinal[0,:])
kFR,cFR = np.polyfit(timeAxis, lnFR, 1)
aFR = math.exp(cFR)
fitFR = np.zeros(tTotal+1)
for i in range(tTotal+1):
	fitFR[i] = aFR * math.exp(kFR * i)
corrFR = np.corrcoef(frFinal[0,:], fitFR)
rsqFR = corrFR[0,1]**2
# Total ALL cells fitting
lnTot = np.log(totFinal[0,:])
kTot,cTot = np.polyfit(timeAxis, lnTot, 1)
aTot = math.exp(cTot)
fitTot = np.zeros(tTotal+1)
for i in range(tTotal+1):
	fitTot[i] = aTot * math.exp(kTot * i)
corrTot = np.corrcoef(totFinal[0,:], fitTot)
rsqTot = corrTot[0,1]**2
print(f"\nThe number of F cells can be fitted by the equation N = {aF}*e^({kF}*t) with R2 = {rsqF}")
print(f"The number of R cells can be fitted by the equation N = {aR}*e^({kR}*t) with R2 = {rsqR}") 
print(f"The number of mutant cells can be fitted by the equation N = {aM}*e^({kM}*t) with R2 = {rsqM}") 
print(f"The total number of live cells (FR) can be fitted by the equation N = {aFR}*e^({kFR}*t) with R2 = {rsqFR}")
print(f"The total number of ALL cells can be fitted by the equation N = {aTot}*e^({kTot}*t) with R2 = {rsqTot}")

# EXPONENTIAL FITTING OF SD OVER TIME  ### NON-MUTANT ADJUSTED
timeAxis = list(range(1,tTotal+1))
# F cells fitting
lnSDf = np.log(fFinal[1,1:])
SDkf,SDcf = np.polyfit(timeAxis, lnSDf, 1)
SDaf = math.exp(SDcf)
fitSDF = np.zeros(tTotal)
for i in range(tTotal):
	fitSDF[i] = SDaf * math.exp(SDkf * i)
corrSDF = np.corrcoef(fFinal[1,1:], fitSDF)
rsqSDF = corrSDF[0,1]**2
# R cells fitting
lnSDr = np.log(rFinal[1,1:])
SDkr,SDcr = np.polyfit(timeAxis, lnSDr, 1)
SDar = math.exp(SDcr)
fitSDR = np.zeros(tTotal)
for i in range(tTotal):
	fitSDR[i] = SDar * math.exp(SDkr * i)
corrSDR = np.corrcoef(rFinal[1,1:], fitSDR)
rsqSDR = corrSDR[0,1]**2
# Total cells fitting
lnSDfr = np.log(frFinal[1,1:])
SDkfr,SDcfr = np.polyfit(timeAxis, lnSDfr, 1)
SDafr = math.exp(SDcfr)
fitSDFR = np.zeros(tTotal)
for i in range(tTotal):
	fitSDFR[i] = SDafr * math.exp(SDkfr * i)
corrSDFR = np.corrcoef(frFinal[1,1:], fitSDFR)
rsqSDFR = corrSDFR[0,1]**2
print(f"\nThe SD for F cells can be fitted by the equation SD = {SDaf}*e^({SDkf}*t) with R2 = {rsqSDF}")
print(f"The SD for R cells can be fitted by the equation SD = {SDar}*e^({SDkr}*t) with R2 = {rsqSDR}") 
print(f"The SD for total cells can be fitted by the equation SD = {SDafr}*e^({SDkfr}*t) with R2 = {rsqSDFR}") 

# EXTRACT THE CV VALUE WHEN IT CONVERGES    ### NON-MUTANT ADJUSTED
timeRange = round(tTotal*0.8) # Assume we only use the final 20% time for fitting, which is about the time when the curve converges
timeAxis = list(range(timeRange,tTotal))
mFCV,cFCV = np.polyfit(timeAxis, fCV[timeRange:tTotal], 1) # Linear fitting of F cells CV (y = mx + c)
mRCV,cRCV = np.polyfit(timeAxis, rCV[timeRange:tTotal], 1) # Linear fitting of R cells CV (y = mx + c)
mFRCV,cFRCV = np.polyfit(timeAxis, frCV[timeRange:tTotal], 1) # Linear fitting of total cells CV (y = mx + c)
# Print the values
print(f"\nThe CV of F cells can be fitted by the equation y = {mFCV}t + {cFCV}")
print(f"The CV of R cells can be fitted by the equation y = {mRCV}t + {cRCV}") 
print(f"The CV of total cells can be fitted by the equation y = {mFRCV}t + {cFRCV}")

# EXTRACT THE VALUE OF THE RATIO WHEN IT CONVERGES  	### NON-MUTANT ADJUSTED
timeRange = round(tTotal*0.8) # Assume we only use the final 20% time for fitting, which is about the time when the curve converges
timeAxis = list(range(timeRange,tTotal))
mF,cF = np.polyfit(timeAxis, fRatio[0,timeRange:tTotal], 1) # Linear fitting of F cells proportion (y = mx + c)
mR,cR = np.polyfit(timeAxis, rRatio[0,timeRange:tTotal], 1) # Linear fitting of R cells proportion (y = mx + c)
mFR,cFR = np.polyfit(timeAxis, ratio[0,timeRange:tTotal], 1) # Linear fitting of F:R cells ratio (y = mx + c)
# Print the values
print(f"\nThe proportion of F cells can be fitted by the equation y = {mF}t + {cF}")
print(f"The proportion of R cells can be fitted by the equation y = {mR}t + {cR}") 
print(f"The ratio of F:R cells can be fitted by the equation y = {mFR}t + {cFR}") 


### GRAPH PLOTTING

# PLOT THE GRAPH WITH SD
timeAxis = list(range(0,tTotal+1)) # Set the time axis
plt.plot(timeAxis, fFinal[0,:], "r", label="F cells") # Plot total number of F cells over time
plt.plot(timeAxis, rFinal[0,:], "b", label="R cells") # Plot total number of R cells over time
plt.plot(timeAxis, frFinal[0,:], "g", label="Total cells") # Plot total number of cells over time
plt.plot(timeAxis, mFinal[0,:], "k", label="Mutant cells") # Plot total number of mutant cells over time
plt.errorbar(timeAxis, fFinal[0,:], yerr=fFinal[1,:], color="red")
plt.errorbar(timeAxis, rFinal[0,:], yerr=rFinal[1,:], color="blue")
plt.errorbar(timeAxis, frFinal[0,:], yerr=frFinal[1,:], color="green")
plt.errorbar(timeAxis, mFinal[0,:], yerr=mFinal[1,:], color="black")
plt.title(f'Number of cells over time in {trials} trials with SD')
plt.legend(loc="upper left")
plt.yscale('log')
plt.xlabel('Time (min)')
plt.ylabel('Number of cells')
plt.show()

# PLOT THE GRAPH WITH 95% RANGE
timeAxis = list(range(0,tTotal+1)) # Set the time axis
plt.plot(timeAxis, fFinal[0,:], "r", label="F cells") # Plot total number of F cells over time
plt.plot(timeAxis, rFinal[0,:], "b", label="R cells") # Plot total number of R cells over time
plt.plot(timeAxis, frFinal[0,:], "g", label="Total cells") # Plot total number of cells over time
plt.plot(timeAxis, mFinal[0,:], "k", label="Mutant cells") # Plot total number of mutant cells over time
plt.errorbar(timeAxis, fFinal[0,:], yerr=[fFinal[2,:],fFinal[3,:]], color="red")
plt.errorbar(timeAxis, rFinal[0,:], yerr=[rFinal[2,:],rFinal[3,:]], color="blue")
plt.errorbar(timeAxis, frFinal[0,:], yerr=[frFinal[2,:],frFinal[3,:]], color="green")
plt.errorbar(timeAxis, mFinal[0,:], yerr=[mFinal[2,:],mFinal[3,:]], color="black")
plt.title(f'Number of cells over time in {trials} trials with 95% range')
plt.legend(loc="upper left")
plt.yscale('log')
plt.xlabel('Time (min)')
plt.ylabel('Number of cells')
plt.show()

# PLOT THE SD GRAPH			### NON-MUTANT ADJUSTED
timeAxis = list(range(0,tTotal+1)) # Set the time axis
plt.plot(timeAxis, fFinal[1,:], "r", label="F cells") # Plot SD of F cells over time
plt.plot(timeAxis, rFinal[1,:], "b", label="R cells") # Plot SD of R cells over time
plt.plot(timeAxis, frFinal[1,:], "g", label="Total cells") # Plot SD of total cells over time
plt.title(f'SD over time in {trials} trials')
plt.legend(loc="upper left")
plt.xlabel('Time (min)')
plt.ylabel('SD')
plt.show()

# PLOT THE LOG(SD) GRAPH
timeAxis = list(range(0,tTotal+1)) # Set the time axis
plt.plot(timeAxis, fFinal[1,:], "r", label="F cells") # Plot SD of F cells over time
plt.plot(timeAxis, rFinal[1,:], "b", label="R cells") # Plot SD of R cells over time
plt.plot(timeAxis, frFinal[1,:], "g", label="Total cells") # Plot SD of total cells over time
plt.title(f'SD over time in {trials} trials in log scale')
plt.legend(loc="upper left")
plt.xlabel('Time (min)')
plt.ylabel('SD')
plt.yscale('log')
plt.show()

# PLOT THE CV GRAPH			### NON-MUTANT ADJUSTED
timeAxis = list(range(0,tTotal+1)) # Set the time axis
plt.plot(timeAxis, fCV, "r", label="F cells") # Plot CV of F cells over time
plt.plot(timeAxis, rCV, "b", label="R cells") # Plot CV of R cells over time
plt.plot(timeAxis, frCV, "g", label="Total cells") # Plot CV of total cells over time
plt.title(f'Variation of CV over time')
plt.legend(loc="upper left")
plt.xlabel('Time (min)')
plt.ylabel('CV')
plt.show()

# PLOT THE RATIO GRAPH			### NON-MUTANT ADJUSTED
timeAxis = list(range(0,tTotal+1)) # Set the time axis
plt.plot(timeAxis, ratio[0,:], "r", label="F:R Ratio") # Plot F:R cell ratio over time
plt.errorbar(timeAxis, ratio[0,:], yerr=ratio[1,:], color="red")
plt.title(f'F:R cell ratio over time in {trials} trials')
plt.legend(loc="upper left")
plt.xlabel('Time (min)')
plt.ylabel('F:R ratio')
ymin, ymax = plt.ylim()
plt.ylim(ymin * 0.9, ymax * 1.1)
plt.show()

# PLOT THE DOUBLE RATIO GRAPH
timeAxis = list(range(0,tTotal+1)) # Set the time axis
plt.plot(timeAxis, fRatio[0,:], "r", label="F Proportion") # Plot F cell proportion over time
plt.plot(timeAxis, rRatio[0,:], "b", label="R Proportion") # Plot R cell proportion over time
plt.errorbar(timeAxis, fRatio[0,:], yerr=fRatio[1,:], color="red")
plt.errorbar(timeAxis, rRatio[0,:], yerr=rRatio[1,:], color="blue")
plt.title(f'Proportion of F and R cells over time in {trials} trials')
plt.legend(loc="upper left")
plt.xlabel('Time (min)')
plt.ylabel('Proportion')
plt.ylim(0,1)
plt.show()

# PLOT THE TRIPLE RATIO GRAPH WITH MUTANTS		### NON-MUTANT ADJUSTED
timeAxis = list(range(0,tTotal+1)) # Set the time axis
plt.plot(timeAxis, fFinal[0,:]/totFinal[0,:], "r", label="F Proportion") # Plot F cell proportion over time
plt.plot(timeAxis, rFinal[0,:]/totFinal[0,:], "b", label="R Proportion") # Plot R cell proportion over time
plt.plot(timeAxis, mFinal[0,:]/totFinal[0,:], "k", label="Mutant Proportion") # Plot R cell proportion over time
#plt.errorbar(timeAxis, fRatio[0,:], yerr=fRatio[1,:], color="red")
#plt.errorbar(timeAxis, rRatio[0,:], yerr=rRatio[1,:], color="blue")
plt.title(f'Proportion of F, R, and mutant cells over time in {trials} trials')
plt.legend(loc="upper left")
plt.xlabel('Time (min)')
plt.ylabel('Proportion')
plt.ylim(0,1)
plt.show()